
TCP = dofile("tcpsock.lua")

local enabled = (gkini.ReadInt("*rumbler", "enabled", 0) == 1)
local doengine = (gkini.ReadInt("*rumbler", "doengine", 1) == 1)
local dohit = (gkini.ReadInt("*rumbler", "dohit", 1) == 1)


local PORT = 44455

local enginerumble_timer = Timer()

local conn

local lastrumble

local minrumblespeed
local turbospeed


local function enginerumble_timer_cb()
	if not enabled or not IsConnected() then
		return
	end
		
	local speed = GetActiveShipSpeed()
	if GetActiveShipID() and speed > minrumblespeed then
		local strength = math.floor((math.max((speed - minrumblespeed) / (turbospeed - minrumblespeed)) * 20)) + 10
		rumbler_send(strength, 500)
	end
	enginerumble_timer:SetTimeout(500, enginerumble_timer_cb)
end

local function enginerumble_start()
	if not enabled or not doengine then
		return
	end

	local shipid = GetActiveShipID()
	if not shipid then
		return
	end

	turbospeed = tonumber(GetInventoryItemExtendedDesc(shipid):gmatch("Turbo max speed: (.*)m/s")())
	minrumblespeed = turbospeed * 0.6

	enginerumble_timer_cb()
end

local function enginerumble_stop()
	enginerumble_timer:Kill()
end


local function rumbler_disconnect()
	enginerumble_stop()
	enabled = false
	
	if conn then
		conn.tcp:Disconnect()
	end
end

local function rumbler_connect()
	rumbler_disconnect()

	enabled = true

	local function disconn_cb()
		print("disconnected from rumbleserver")
		enabled = false
		enginerumble_stop()
		conn = nil
	end

	local function conn_cb(conn2, err)
		if conn2 then
			print("connected to rumbleserver")
			enginerumble_start()
		else
			print("connect to rumbleserver failed: "..tostring(err))
			enabled = false
			conn = nil
		end
	end

	local function line_cb()
	end

	conn = TCP.make_client("127.0.0.1", PORT, conn_cb, line_cb, disconn_cb)
end


function rumbler_send(strength, duration)
	if not lastrumble or gkmisc.DiffTime(lastrumble, gkmisc.GetGameTime()) > 100 then
		if conn then
			conn:Send(string.format("1 %i %i\n", strength, duration))
		else
			print("not connected to rumbleserver")
		end
		lastrumble = gkmisc.GetGameTime()
	end
end



local function en_start_cb()
	enginerumble_start()
end
RegisterEvent(en_start_cb, "SHIP_SPAWNED")


local function en_stop_cb()
	enginerumble_stop()
end
RegisterEvent(en_stop_cb, "ENTERING_STATION")
RegisterEvent(en_stop_cb, "PLAYER_DIED")
RegisterEvent(en_stop_cb, "WARP_OUT")


local function hit_cb(ev, player, charid, objectid, damage)
	if not enabled or not dohit then
		return
	end

	enginerumble_timer:SetTimeout(200)
	rumbler_send(100, 200)
end
RegisterEvent(hit_cb, "PLAYER_GOT_HIT")


local function unload_cb()
	rumbler_disconnect()
end
RegisterEvent(unload_cb, "UNLOAD_INTERFACE")



local function rumbler_connect_cb()
	rumbler_connect()
end
RegisterUserCommand("rumbler_connect", rumbler_connect_cb)


local function rumbler_disconnect_cb()
	rumbler_disconnect()
end
RegisterUserCommand("rumbler_disconnect", rumbler_disconnect_cb)


local function rumbler_enable_cb(_, args)
	if args then
		if args[1] == "on" then
			enabled = true
			rumbler_connect()
		else
			enabled = false
			rumbler_disconnect()
		end
		gkini.WriteInt("*rumbler", "enabled", enabled and 1 or 0)
	else
		print("usage rumbler_enable on|off")
	end
	print("rumbler enabled: "..tostring(enabled))
end
RegisterUserCommand("rumbler_enable", rumbler_enable_cb)


local function rumbler_hit_cb(_, args)
	if args then
		dohit = (args[1] == "on")
		gkini.WriteInt("*rumbler", "dohit", dohit and 1 or 0)
	else
		print("usage rumbler_hit on|off")
	end
	
	print("rumble on hit enabled: "..tostring(dohit))
end
RegisterUserCommand("rumbler_hit", rumbler_hit_cb)


local function rumbler_engine_cb(_, args)
	if args then
		if args[1] == "on" then
			doengine = true
			enginerumble_start()
		else
			doengine = false
			enginerumble_stop()
		end
		gkini.WriteInt("*rumbler", "doengine", doengine and 1 or 0)
	else
		print("usage rumbler_engine on|off")
	end
	
	print("engine rumble enabled: "..tostring(doengine))
end
RegisterUserCommand("rumbler_engine", rumbler_engine_cb)


if enabled then
	rumbler_connect()
end
